import { getJSONResponse } from '../utils/util'

export const rootAPI = 'http://129.241.90.187:1337'

// sourceURL is on the form /datasourcees/{id}, /processors/{id} or /topics/{id}
export async function getOutputNames (sourceUrl) {
  const sourceJSON = await getJSONResponse(rootAPI + sourceUrl)
  return sourceJSON.output_names || []
}

export async function fetchFMUs () {
  return getJSONResponse(rootAPI + /fmus/)
}

export async function fetchBlueprints () {
  return getJSONResponse(rootAPI + '/blueprints/')
}

export async function fetchTopics () {
  return getJSONResponse(rootAPI + '/topics/')
}

export async function subscribeToSource (sourceId) {
  await fetch(rootAPI + '/topics/' + sourceId + '/subscribe', { credentials: 'include' })
}

export async function unSubscribeSource (sourceId) {
  await fetch(rootAPI + '/topics/' + sourceId + '/unsubscribe', { credentials: 'include' })
}

export async function createDataSource (formData) {
  const createLink = rootAPI + '/datasources/create'
  return fetch(createLink, {
    method: 'POST',
    body: formData
  })
}

export async function createProcessor (formData) {
  const createLink = rootAPI + '/processors/create'
  return fetch(createLink, {
    method: 'POST',
    body: formData
  })
}

export async function startProcessorRequest (formData) {
  return fetch(rootAPI + '/processors/start', {
    method: 'POST',
    body: formData
  })
}

// Sends request to change inputs or outputs of running process,
// endPoint is either 'inputs' or 'outputs'
export async function editProcessorIOs (processorID, endPoint, formData) {
  const link = rootAPI + '/processors/' + processorID + '/' + endPoint
  return fetch(link, {
    method: 'post',
    body: formData
  })
}

export async function fetchDataSources () {
  const dataSourcesResponse = await getJSONResponse(rootAPI + '/datasources/')
  return dataSourcesResponse || []
}

export async function startDataSource (datasourceID) {
  const startLink = rootAPI + '/datasources/' + datasourceID + '/start'
  return fetch(startLink)
}

export async function fetchBlueprint (blueprintID) {
  return getJSONResponse(rootAPI + '/blueprints/' + blueprintID)
}
