// Returns an processor object with two lists: Inputs and outputs
import { deepCopy } from '../utils/util'

export function extractProcessorIOs (processorJSON) {
  if (processorJSON.inputs === undefined && processorJSON.outputs === undefined) {
    return false
  }
  const selectedMeasurementRefs = processorJSON.measurement_refs
  const inputs = processorJSON.inputs.map((input, index) => ({
    input_ref: index,
    name: input.name,
    measurement_ref: selectedMeasurementRefs[index] || -1,
    measurement_proportion: 1
  }))
  const matrixOutputRefs = processorJSON.matrix_outputs
  const outputRefs = processorJSON.output_refs
  let scalarOutputs = processorJSON.outputs.map((output, index) => ({
    id: index,
    name: output.name,
    selected: outputRefs.includes(index)
  }))
  if (matrixOutputRefs !== undefined) {
    const allOutputs = deepCopy(scalarOutputs)
    const matrixOutputs = Object.entries(matrixOutputRefs).map((matrixOutput) => {
      const matrixOutputIndices = matrixOutput[1]
      return {
        name: matrixOutput[0] + '_matrix',
        selected: outputRefs.includes(matrixOutputIndices[0]),
        matrixOutputNames: allOutputs.filter((output, index) => {
          if (matrixOutputIndices.includes(index)) {
            scalarOutputs.splice(index)
            return true
          }
        })
      }
    })
    return { inputs: inputs, outputs: scalarOutputs.concat(matrixOutputs) }
  }
}

export function getInitDocs (docs) {
  let newInitDocs = {}
  let docArray = docs.init_docs.split(':')
  if (docArray[0].split(' ')[0] !== 'param') {
    docArray = docArray.slice(1)
  }
  let currentParamName = ''
  docArray.forEach(line => {
    let tempArray = line.split(' ')
    if (tempArray[0] === 'param') {
      currentParamName = tempArray[1]
    } else {
      newInitDocs[currentParamName] = line
    }
  })
  return newInitDocs
}
