export function createProcessorInputFormData (inputs) {
  let formData = new FormData()
  const insertedInputs = inputs.filter(input => input.measurement_ref !== -1)
  if (insertedInputs.length === 0) {
    // No inputs have any reference to a datachannel
    return false
  }
  insertedInputs.forEach(input => {
    formData.append('input_ref', input.input_ref)
    formData.append('measurement_ref', input.measurement_ref)
    formData.append('measurement_proportion', input.measurement_proportion)
  })
  return formData
}

export function createProcessorOutputFormData (outputs, formData) {
  if (formData === undefined) {
    formData = new FormData()
  }
  for (let i = 0; i < outputs.length; i++) {
    const matrixOutputs = outputs[i].matrixOutputNames
    if (matrixOutputs === undefined) {
      formData.append('output_ref', outputs[i].id)
    } else {
      matrixOutputs.forEach(output => {
        formData.append('output_ref', output.id)
      })
    }
  }
  return formData
}

export function createProcessorFormData (processorId, selectedBlueprint, initParams, source) {
  let formData = new FormData()
  formData.append('id', processorId)
  formData.append('blueprint', selectedBlueprint)
  formData.append('init_params', JSON.stringify(initParams))
  formData.append('topic', source)
  formData.append('min_output_interval', '0.01')
  return formData
}
