// Ceetron Cloud Components, version 2.4.1-08fd73d4
// Copyright (C) 2017, Ceetron AS, https://ceetron.com/
// Use of this file requires a distribution license from Ceetron AS.
// The contents of this file may not be copied or duplicated in any form, in whole or in part, 
// without the prior written permission of Ceetron AS.

// Type definitions for Ceetron Cloud Components, version: 2.4.1-08fd73d4 

// Make cee available as a global for non-module code.
export as namespace cee;


/**
 *
 */
export class BoundingBox {
    readonly minimum: Vec3;
    readonly maximum: Vec3;
    constructor(minimum: Vec3Like, maximum: Vec3Like);
    isValid(): boolean;
    getCenter(): Vec3;
    getExtent(): Vec3;
}

/**
 *
 */
export enum ProjectionType {
    PERSPECTIVE = 0,
    ORTHO = 1,
}

/**
 *
 */
export class Camera {
    private constructor();
    readonly fieldOfViewYDeg: number;
    readonly frontPlaneFrustumHeight: number;
    readonly nearPlane: number;
    readonly farPlane: number;
    readonly projectionType: ProjectionType;
    setProjectionAsPerspective(fieldOfViewYDeg: number, nearPlane: number, farPlane: number): void;
    setProjectionAsOrtho(height: number, nearPlane: number, farPlane: number): void;
    setClipPlanesFromBoundingBox(boundingBox: BoundingBox, minNearPlaneDistance: number): void;
    getPosition(): Vec3;
    getDirection(): Vec3;
    getUp(): Vec3;
    setViewpoint(eye: Vec3Like, direction: Vec3Like, up: Vec3Like): void;
    setFromLookAt(eye: Vec3Like, center: Vec3Like, up: Vec3Like): void;
    fitView(boundingBox: BoundingBox, dir: Vec3Like, up: Vec3Like, coverageFactor?: number): void;
    zoomToBoundingBox(boundingBox: BoundingBox): void;
    fitViewOrtho(boundingBox: BoundingBox, eyeDist: number, dir: Vec3Like, up: Vec3Like, coverageFactor?: number): void;
    enableAutoClipMinimumNearDistance(minNearDistance: number): void;
    enableAutoClipFixedNearDistance(fixedNearDistance: number): void;
    disableAutoClip(): void;
}

/**
 *
 */
export class Clipping {
    private constructor();
    requiredNumberOfPlaneRejectsToClip: number;
    readonly planeCount: number;
    getPlaneArray(): ReadonlyArray<Plane>;
    getPlaneAt(index: number): Plane;
    addPlane(plane: Plane): void;
    setPlaneAt(index: number, plane: Plane): void;
    removePlaneAt(index: number): void;
    removeAllPlanes(): void;
}

/**
 *
 */
export class CloudSession {
    constructor();
    readonly version: string;
    readonly viewerCount: number;
    getViewerArray(): ReadonlyArray<Viewer>;
    addViewer(canvas: HTMLCanvasElement, viewerOptions?: ViewerOptions): Viewer;
    getViewerAt(index: number): Viewer;
    deleteViewerAt(index: number): void;
    deleteAllViewers(): void;
    destroySession(): void;
    handleAnimationFrameCallback(highResTimestamp: number): boolean;
    addModelToProcessingSet(model: ProcessableModel): void;
    removeModelFromProcessingSet(model: ProcessableModel): void;
}

/**
 *
 */
export interface Color3Like {
    readonly r: number;
    readonly g: number;
    readonly b: number;
}

/**
 *
 */
export class Color3 {
    readonly r: number;
    readonly g: number;
    readonly b: number;
    constructor(r: number, g: number, b: number);
    equals(other: Color3Like): boolean;
    static from(color: Color3Like): Color3;
    static fromColor4(color: Color4Like): Color3;
    static fromArray(arr: ArrayLike<number>): Color3;
}

/**
 *
 */
export interface Color4Like {
    readonly r: number;
    readonly g: number;
    readonly b: number;
    readonly a: number;
}

/**
 *
 */
export class Color4 {
    readonly r: number;
    readonly g: number;
    readonly b: number;
    readonly a: number;
    constructor(r: number, g: number, b: number, a: number);
    equals(other: Color4Like): boolean;
    static from(color: Color4Like): Color4;
}

/**
 *
 */
export enum ColorTable {
    NORMAL = 0,
    BLACK_WHITE = 1,
    BLUE_RED = 2,
    BLUE_GREEN = 3,
    YELLOW_RED = 4,
    GREEN_YELLOW_RED = 5,
    RED_YELLOW = 6,
    THERMAL_1 = 7,
    THERMAL_2 = 8,
    THERMAL_3 = 9,
    METAL_CASTING = 10,
}

/**
 *
 */
export class ColorTableFactory {
    static color3TableArray(colorTable: ColorTable, colorCount: number): Color3Like[];
    static color4TableArray(colorTable: ColorTable, colorCount: number): Color4Like[];
    static interpolateColorArray(colorArray: Color3Like[], targetColorCount: number): Color3Like[];
    static interpolateColor4Array(colorArray: Color4Like[], targetColorCount: number): Color4Like[];
    static partColorArray(colorCount: number): Color3Like[];
}

/**
 *
 */
export class Logger {
    static error: any;
    static warn: any;
    static info: (_msg: string) => void;
    static debug: (_msg: string) => void;
    static enableInfo(enable?: boolean): void;
    static enableDebug(enable?: boolean): void;
}

/**
 *
 */
export class Mat4 {
    constructor();
    isIdentity(): boolean;
    getInverse(): Mat4;
    getRowCol(row: number, col: number): number;
    getAsArray(): ArrayLike<number>;
    static multiply(a: Mat4, b: Mat4): Mat4;
    static translatePreMultiply(M: Mat4, tv: Vec3Like): Mat4;
    static translatePostMultiply(M: Mat4, tv: Vec3Like): Mat4;
    static fromArray(array: ArrayLike<number>): Mat4;
    static fromRotation(axis: Vec3Like, angle: number): Mat4;
    static fromCoordSystemAxes(xAxis: Vec3Like, yAxis: Vec3Like, zAxis: Vec3Like): Mat4;
    static fromTranslation(trans: Vec3Like): Mat4;
    static fromScaling(scale: Vec3Like): Mat4;
    static fromElements(m00: number, m01: number, m02: number, m03: number, m10: number, m11: number, m12: number, m13: number, m20: number, m21: number, m22: number, m23: number, m30: number, m31: number, m32: number, m33: number): Mat4;
}

/**
 *
 */
export abstract class Model {
    readonly abstract name: string;
    abstract getBoundingBox(): BoundingBox;
}

/**
 *
 */
export enum NavigationType {
    NONE = 0,
    PAN = 1,
    ZOOM = 2,
    WALK = 3,
    ROTATE = 4,
    ROTATE_SCREEN_AXES = 5,
    ROLL = 6,
}

/**
 *
 */
export interface NavigationMouseButtonConfig {
    noModifier: NavigationType;
    shiftModifier: NavigationType;
    ctrlOrCmdModifier: NavigationType;
}

/**
 *
 */
export interface NavigationConfig {
    readonly leftMouseButton: NavigationMouseButtonConfig;
    readonly rightMouseButton: NavigationMouseButtonConfig;
    readonly middleMouseButton: NavigationMouseButtonConfig;
    readonly leftAndRightMouseButtons: NavigationMouseButtonConfig;
    wheelOrPinch: NavigationType;
    oneFinger: NavigationType;
    twoFinger: NavigationType;
    invertWalkZoom: boolean;
    invertWheel: boolean;
}

/**
 *
 */
export class Navigation {
    readonly config: NavigationConfig;
    rotationPoint: Vec3Like;
    minWalkTargetDistance: number;
    rotationSensitivity: number;
}

/**
 *
 */
export class Overlay {
    private constructor();
    navigationCubeVisible: boolean;
    configureNavigationCube(faceNames: string[], faceColors?: Color3Like[], axisLabels?: string[]): void;
    navigationCubeHomeDirection: Vec3;
    navigationCubeHomeUp: Vec3;
    infoBoxVisible: boolean;
    stepTextVisible: boolean;
    autoAppendModelInfoToInfoBox: boolean;
    modelColorLegendsVisible: boolean;
    maximumLegendHeight: number;
    ceetronLogoVisible: boolean;
    setInfoBoxContent(content: string): void;
    setStepTextContent(content: string): void;
    addCustomColorLegendForScalarMapper(scalarMapper: ScalarMapperContinuousDomain, legendTitle: string, resultId: number): void;
    addCustomColorLegend(legend: OverlayColorLegendContinuousDomain): void;
    addCustomCategoryLegend(categories: string[], colors: ArrayLike<Color3Like>, legendTitle: string, resultId: number): void;
    removeAllCustomLegends(): void;
}

/**
 *
 */
export class OverlayColorLegendContinuousDomain {
    constructor(resultId: number);
    title: string;
    textColor: Color3Like;
    fontSizePixels: number;
    legendBarWidth: number;
    lineColor: Color3Like;
    forceUniformTickMarkSpacing: boolean;
    drawTickMarksOnTopOfLegendBar: boolean;
    showTickMarkLabels: boolean;
    setCustomLabels(labels: string[], relativePositions: number[]): void;
    clearCustomLabels(): void;
    setupFromScalarMapper(scalarMapper: ScalarMapperContinuousDomain): void;
    setTickValues(values: ArrayLike<number>, scalarMapper: ScalarMapperContinuousDomain): void;
}

/**
 *
 */
export class Plane {
    constructor(A: number, B: number, C: number, D: number);
    readonly A: number;
    readonly B: number;
    readonly C: number;
    readonly D: number;
    getDistance(point: Vec3Like): number;
    getDistanceSquared(point: Vec3Like): number;
    getNormal(): Vec3Like;
    static fromPointAndNormal(point: Vec3Like, normal: Vec3Like): Plane;
    static fromPoints(p1: Vec3Like, p2: Vec3Like, p3: Vec3Like): Plane;
}

/**
 *
 */
export interface ProcessableModel {
}

/**
 *
 */
export class Range {
    readonly minimum: number;
    readonly maximum: number;
    constructor(min: number, max: number);
    isValid(): boolean;
}

/**
 *
 */
export class Ray {
    origin: Vec3Like;
    direction: Vec3Like;
    constructor(origin: Vec3Like, direction: Vec3Like);
    clone(): Ray;
    transform(matrix: Mat4): void;
    pointDistanceSquared(point: Vec3Like): number;
}

/**
 *
 */
export class RaySphereIntersector {
    static findClosestIntersectedSphereToRay(ray: Ray, radius: number, pointsArr: ArrayLike<number>): number;
    static findAllIntersectedSpheresOrderedByDistanceToRay(ray: Ray, radius: number, pointsArr: ArrayLike<number>): {
        index: number;
        l2Distance: number;
    }[];
}

/**
 *
 */
export interface ScalarMapper {
    mapToColor(scalarValue: number): Color4Like;
    mapToTextureCoordinate(scalarValue: number): Vec2Like;
    createTexture(): Texture;
}

/**
 *
 */
export class ScalarMapperContinuous extends ScalarMapperContinuousDomain {
    constructor();
    readonly rangeMinimum: number;
    readonly rangeMaximum: number;
    readonly logarithmic: boolean;
    setRange(min: number, max: number): void;
    setRangeLogarithmic(min: number, max: number): void;
    setColors(colors: Color4Like[]): void;
    setColorsFromColorTable(colorTable: ColorTable): void;
    setColorsAtValues(colorArr: ArrayLike<Color4Like>, valueArr: ArrayLike<number>): void;
    mapToColor(scalarValue: number): Color4Like;
    mapToTextureCoordinate(scalarValue: number): Vec2Like;
    createTexture(): Texture;
    normalizedValue(domainValue: number): number;
    domainValue(normalizedPosition: number): number;
    version(): number;
    tickValuesHint(): ArrayLike<number>;
}

/**
 *
 */
export abstract class ScalarMapperContinuousDomain implements ScalarMapper {
    abstract mapToColor(scalarValue: number): Color4Like;
    abstract mapToTextureCoordinate(scalarValue: number): Vec2Like;
    abstract createTexture(): Texture;
    abstract normalizedValue(domainValue: number): number;
    abstract domainValue(normalizedPosition: number): number;
    abstract version(): number;
    abstract tickValuesHint(): ArrayLike<number>;
    aboveRangeColor: Color4Like;
    belowRangeColor: Color4Like;
    undefinedColor: Color4Like;
}

/**
 *
 */
export class ScalarMapperContinuousPiecewise extends ScalarMapperContinuousDomain {
    constructor();
    readonly rangeMinimum: number;
    readonly rangeMaximum: number;
    setup(colorsFromToArray: ArrayLike<Color4Like>, tickValuesArray: ArrayLike<number>): void;
    mapToColor(scalarValue: number): Color4Like;
    mapToTextureCoordinate(scalarValue: number): Vec2Like;
    createTexture(): Texture;
    normalizedValue(domainValue: number): number;
    domainValue(normalizedPosition: number): number;
    version(): number;
    tickValuesHint(): ArrayLike<number>;
}

/**
 *
 */
export class ScalarMapperFilledContours extends ScalarMapperContinuousDomain {
    constructor();
    readonly rangeMinimum: number;
    readonly rangeMaximum: number;
    setColorsAtValues(colorArr: ArrayLike<Color4Like>, valueArr: ArrayLike<number>): void;
    mapToColor(scalarValue: number): Color4Like;
    mapToTextureCoordinate(scalarValue: number): Vec2Like;
    createTexture(): Texture;
    normalizedValue(domainValue: number): number;
    domainValue(normalizedPosition: number): number;
    version(): number;
    tickValuesHint(): ArrayLike<number>;
}

/**
 *
 */
export class ScalarMapperFilledContoursUniform extends ScalarMapperContinuousDomain {
    constructor();
    readonly rangeMinimum: number;
    readonly rangeMaximum: number;
    colorArray: ArrayLike<Color4Like>;
    setRange(min: number, max: number): void;
    setRangeLogarithmic(min: number, max: number): void;
    mapToColor(scalarValue: number): Color4Like;
    mapToTextureCoordinate(scalarValue: number): Vec2Like;
    createTexture(): Texture;
    normalizedValue(domainValue: number): number;
    domainValue(normalizedPosition: number): number;
    version(): number;
    tickValuesHint(): ArrayLike<number>;
}

/**
 *
 */
export class SnapPointPicker {
    constructor(view: View, ray: Ray, radius: number);
    findClosestPoint(pointsArr: ArrayLike<number>): number;
}

/**
 *
 */
export enum TextureWrapMode {
    REPEAT = 0,
    CLAMP_TO_EDGE = 1,
}

/**
 *
 */
export enum TextureMinFilter {
    NEAREST = 0,
    LINEAR = 1,
    NEAREST_MIPMAP_NEAREST = 2,
    NEAREST_MIPMAP_LINEAR = 3,
    LINEAR_MIPMAP_NEAREST = 4,
    LINEAR_MIPMAP_LINEAR = 5,
}

/**
 *
 */
export enum TextureMagFilter {
    NEAREST = 0,
    LINEAR = 1,
}

/**
 *
 */
export class TextureOptions {
    generateMipmaps?: boolean;
    wrapMode?: TextureWrapMode;
    minFilter?: TextureMinFilter;
    magFilter?: TextureMagFilter;
}

/**
 *
 */
export class Texture {
    private constructor();
    static fromImageData(imageData: ImageData, textureOptions?: TextureOptions): Texture;
    static fromHTMLImageElement(htmlImageElement: HTMLImageElement, textureOptions?: TextureOptions): Texture;
}

/**
 *
 */
export interface Vec2Like {
    readonly x: number;
    readonly y: number;
}

/**
 *
 */
export class Vec2 {
    readonly x: number;
    readonly y: number;
    constructor(x: number, y: number);
    equals(other: Vec2Like): boolean;
    getLengthSquared(): number;
    getLength(): number;
    getNormalized(): Vec2;
    getNegated(): Vec2;
    static add(a: Vec2Like, b: Vec2Like): Vec2;
    static sub(a: Vec2Like, b: Vec2Like): Vec2;
    static dot(a: Vec2Like, b: Vec2Like): number;
    static scale(vec: Vec2Like, factor: number): Vec2;
    static negate(vec: Vec2Like): Vec2;
    static from(vec: Vec2Like): Vec2;
    static fromArray(arr: ArrayLike<number>): Vec2;
}

/**
 *
 */
export interface Vec3Like {
    readonly x: number;
    readonly y: number;
    readonly z: number;
}

/**
 *
 */
export class Vec3 {
    readonly x: number;
    readonly y: number;
    readonly z: number;
    constructor(x: number, y: number, z: number);
    equals(other: Vec3Like): boolean;
    getLengthSquared(): number;
    getLength(): number;
    getNormalized(): Vec3;
    getNegated(): Vec3;
    getPointDistanceSquared(otherPoint: Vec3Like): number;
    getPointDistance(otherPoint: Vec3): number;
    static add(a: Vec3Like, b: Vec3Like): Vec3;
    static sub(a: Vec3Like, b: Vec3Like): Vec3;
    static dot(a: Vec3Like, b: Vec3Like): number;
    static cross(a: Vec3Like, b: Vec3Like): Vec3;
    static scale(vec: Vec3Like, factor: number): Vec3;
    static negate(vec: Vec3Like): Vec3;
    static transformPoint(p: Vec3Like, matrix: Mat4): Vec3;
    static transformVector(p: Vec3Like, matrix: Mat4): Vec3;
    static from(vec: Vec3Like): Vec3;
    static fromArray(arr: ArrayLike<number>): Vec3;
}

/**
 *
 */
export class View {
    private constructor();
    name: string;
    readonly ownerViewer: Viewer;
    readonly overlay: Overlay;
    readonly camera: Camera;
    readonly clipping: Clipping;
    readonly navigation: Navigation;
    backgroundColor: Color3Like;
    nonHighlightedDimFactor: number;
    haloHighlightColor: Color3Like;
    readonly modelCount: number;
    getModelArray(): ReadonlyArray<Model>;
    addModel(model: Model): void;
    removeModelAt(modelIndex: number): void;
    removeModel(model: Model): void;
    removeAllModels(): void;
    getIndexOfModel(model: Model): number;
    getModelAt(index: number): Model;
    getBoundingBox(): BoundingBox;
    requestRedraw(): void;
    rayIntersect(ray: Ray): Vec3;
    getRayThroughPoint(point: Vec3): Ray;
    setViewport(x: number, y: number, width: number, height: number): void;
}

/**
 *
 */
export class KeyboardModifiers {
    shift: boolean;
    control: boolean;
    alt: boolean;
    meta: boolean;
}

/**
 *
 */
export interface PickHandlerData {
    view: View;
    modifiers: KeyboardModifiers;
    ray: Ray;
}

/**
 *
 */
export interface PickHandler {
    (pickHandlerData: PickHandlerData): void;
}

/**
 *
 */
export interface ColorLegendClickedHandler {
    (resultId: number): void;
}

/**
 *
 */
export interface MouseNavigationChangedHandler {
    (currentNavigationType: NavigationType): void;
}

/**
 *
 */
export class Viewer {
    private constructor();
    readonly pixelScaleFactor: number;
    readonly viewCount: number;
    getViewArray(): ReadonlyArray<View>;
    addView(): View;
    getViewAt(viewIndex: number): View;
    deleteViewAt(viewIndex: number): void;
    deleteAllViews(): void;
    resizeViewer(canvasWidth: number, canvasHeight: number): void;
    requestRedraw(): void;
    flyCameraTo(cameraPos: Vec3, viewDir: Vec3, upVec: Vec3, fov: number, animDuration: number): void;
    setPickHandler(pickHandler: PickHandler): void;
    setColorLegendClickHandler(legendClickHandler: ColorLegendClickedHandler): void;
    setActiveMouseNavigationsChangedHandler(navigationHandler: MouseNavigationChangedHandler): void;
    enableNavigationHandling(enable: boolean): void;
    enableMouseButtonTracking(enable: boolean): void;
    rayFromMouseCoordinate(view: View, clientX: number, clientY: number): Ray;
    rubberBandZoom(view: View, clientX: number, clientY: number, width: number, height: number): void;
    setShowDebugWindow(show: boolean): void;
}

/**
 *
 */
export enum WBOITMode {
    OPAQUE_DEFAULT_FRAMEBUFFER = 0,
    OPAQUE_OFFSCREEN = 1,
}

/**
 *
 */
export enum HighlightMode {
    OFF = 0,
    SIMPLE = 1,
    DEPTH_BASED_DIMMING = 2,
}

/**
 *
 */
export class ViewerOptions {
    enableTransparencyWBOIT?: boolean;
    wboitMode?: WBOITMode;
    highlightMode?: HighlightMode;
    pixelScaleFactor?: number;
    antiAliasing?: boolean;
    preserveDrawingBuffer?: boolean;
}


//
// geo
// ===================================================================================
declare namespace geo {

/**
 *
 */
export class CgeoModelGenerator {
    constructor(cgeoBuffer: ArrayBuffer, geoModel: GeometryModel);
    buildModel(preserveSharpEdges?: boolean): void;
}

/**
 *
 */
export interface HitItem {
    partIndex: number;
    primitiveIndex: number;
    intersectionPoint: Vec3;
}

/**
 *
 */
export enum MeshPrimitiveType {
    LINES = 2,
    TRIANGLES = 3,
}

/**
 *
 */
export interface Mesh {
    readonly primitiveType: MeshPrimitiveType;
    readonly primitiveCount: number;
    readonly vertexCount: number;
    readonly boundingBox: BoundingBox;
    getPrimitiveVertices(primitiveIndex: number): Vec3[];
    rayIntersect(ray: Ray, hitItem?: MeshHitItem): boolean;
}

/**
 *
 */
export class MeshHitItem {
    primitiveIndex: number;
    intersectionPoint: Vec3;
}

/**
 *
 */
export interface OptionalIndexedLinesVertexData {
    textureCoordinates?: ArrayLike<number>;
}

/**
 *
 */
export class MeshIndexedLines implements Mesh {
    constructor(vertices: ArrayLike<number>, indices: ArrayLike<number>, optionalVertexData?: OptionalIndexedLinesVertexData);
    readonly primitiveType: MeshPrimitiveType;
    readonly primitiveCount: number;
    readonly vertexCount: number;
    readonly boundingBox: BoundingBox;
    readonly vertexArray: ArrayLike<number>;
    readonly indexArray: ArrayLike<number>;
    readonly textureCoordinates: ArrayLike<number>;
    getPrimitiveVertices(primitiveIndex: number): Vec3[];
    rayIntersect(ray: Ray, hitItem?: MeshHitItem): boolean;
}

/**
 *
 */
export interface OptionalIndexedTriangleVertexData {
    vertexNormals?: ArrayLike<number>;
    textureCoordinates?: ArrayLike<number>;
}

/**
 *
 */
export class MeshIndexedTriangles implements Mesh {
    constructor(vertices: ArrayLike<number>, indices: ArrayLike<number>, optionalVertexData?: OptionalIndexedTriangleVertexData);
    static constructPreserveSharpEdges(creaseAngleDegrees: number, vertices: ArrayLike<number>, indices: ArrayLike<number>, optionalVertexData?: OptionalIndexedTriangleVertexData): MeshIndexedTriangles;
    readonly primitiveType: MeshPrimitiveType;
    readonly primitiveCount: number;
    readonly vertexCount: number;
    readonly boundingBox: BoundingBox;
    readonly vertexArray: ArrayLike<number>;
    readonly indexArray: ArrayLike<number>;
    readonly textureCoordinates: ArrayLike<number>;
    getPrimitiveVertices(primitiveIndex: number): Vec3[];
    rayIntersect(ray: Ray, hitItem?: MeshHitItem): boolean;
}

/**
 *
 */
export class GeometryModel extends Model {
    constructor();
    name: string;
    transformationMatrix: Mat4;
    readonly partCount: number;
    getPartArray(): ReadonlyArray<Part>;
    addPart(): Part;
    getPartAt(partIndex: number): Part;
    deletePartAt(partIndex: number): void;
    deletePartsAt(partIndicesArr: number[]): void;
    deleteAllParts(): void;
    getBoundingBox(): BoundingBox;
    rayIntersect(ray: Ray): HitItem;
    setColorPerPartFromMapper(scalarMapper: ScalarMapper, perPartResult: ArrayLike<number>): void;
    regionIntersect(x: number, y: number, width: number, height: number, view: View, acceptPartiallyContainedParts: boolean): Part[];
}

/**
 *
 */
export interface RemoteLoaderModelInfo {
    boundingBox: BoundingBox;
}

/**
 *
 */
export class RemoteLoaderStatusInfo {
    totalChunkCount: number;
    receivedChunkCount: number;
    totalPartCount: number;
    receivedPartCount: number;
}

/**
 *
 */
export class RemoteLoaderTimingInfo {
    elapsedTime_ms: number;
}

/**
 *
 */
export interface RemoteLoaderInitializationCompleteCallback {
    (modelInfo: RemoteLoaderModelInfo): void;
}

/**
 *
 */
export interface RemoteLoaderCompleteCallback {
    (error: Error): void;
}

/**
 *
 */
export class GeometryModelRemoteLoader {
    constructor(serverUrl: string);
    configurePreserveSharpEdges(preserveSharpEdges: boolean): void;
    configureLowResModel(targetLowResModel: GeometryModel): void;
    setInitializationCompleteCallback(initializationCompleteCallback?: RemoteLoaderInitializationCompleteCallback): void;
    loadModel(modelKey: string, targetModel: GeometryModel, loadingCompleteCallback?: RemoteLoaderCompleteCallback): void;
    cancelLoading(): void;
    getStatusInfo(): RemoteLoaderStatusInfo;
    getTimingInfo(): RemoteLoaderTimingInfo;
    setCamera(camera: Camera): void;
}

/**
 *
 */
export class Part {
    private constructor();
    mesh: Mesh;
    transformationMatrix: Mat4;
    readonly settings: PartSettings;
    customData: any;
    readonly partIndex: number;
}

/**
 *
 */
export enum EyeLift {
    OFF = 0,
    FACTOR_1 = 1,
    FACTOR_0_1 = 2,
    FACTOR_0_01 = 3,
}

/**
 *
 */
export enum PolygonOffset {
    OFF = 0,
    ON = 1,
}

/**
 *
 */
export class PartSettings {
    private constructor();
    visible: boolean;
    highlight: boolean;
    intersectable: boolean;
    color: Color3Like;
    opacity: number;
    polygonOffset: PolygonOffset;
    eyeLift: EyeLift;
    texture: Texture;
    lineWidth: number;
}
} // geo


//
// mrk
// ===================================================================================
declare namespace mrk {

/**
 *
 */
export interface LabelAppearanceProperties {
    fontFamily: string;
    fontSize: number;
    fontStyle: "normal" | "italic" | "oblique";
    fontVariant: "normal" | "small-caps";
    fontWeight: "normal" | "bold" | "lighter" | "bolder";
    textColor: Color3Like;
    backgroundVisible: boolean;
    backgroundColor: Color3Like;
    borderVisible: boolean;
    borderColor: Color3Like;
    borderWidth: number;
    opacity: number;
    cornerRadius: number;
}

/**
 *
 */
export class LabelAppearance {
    private constructor();
    fontFamily: string;
    fontSize: number;
    fontStyle: "normal" | "italic" | "oblique";
    fontVariant: "normal" | "small-caps";
    fontWeight: "normal" | "bold" | "lighter" | "bolder";
    textColor: Color3Like;
    backgroundVisible: boolean;
    backgroundColor: Color3Like;
    borderVisible: boolean;
    borderColor: Color3Like;
    borderWidth: number;
    opacity: number;
    cornerRadius: number;
    getAsProperties(): LabelAppearanceProperties;
    setFromProperties(props: Partial<LabelAppearanceProperties>): void;
}

/**
 *
 */
export class MarkupModel extends Model {
    constructor();
    name: string;
    readonly partCount: number;
    addIndexedTrianglesPart(): PartIndexedTriangles;
    addTexturedIndexedTrianglesPart(): PartTexturedIndexedTriangles;
    addPointsPart(): PartPoints;
    addLabelsPart(): PartLabels;
    getPartAt(partIndex: number): Part;
    getBoundingBox(): BoundingBox;
    deletePartAt(partIndex: number): void;
    deleteAllParts(): void;
}

/**
 *
 */
export abstract class Part {
    customData: any;
}

/**
 *
 */
export enum EyeLift {
    OFF = 0,
    FACTOR_1 = 1,
    FACTOR_0_1 = 2,
    FACTOR_0_01 = 3,
}

/**
 *
 */
export class PartIndexedTriangles extends Part {
    private constructor();
    setMeshData(vertices: ArrayLike<number>, indices: ArrayLike<number>): void;
    color: Color3Like;
    eyeLift: EyeLift;
}

/**
 *
 */
export class PartLabels extends Part {
    private constructor();
    readonly labelCount: number;
    add(text: string, pos: Vec3): void;
    addWithOffsetDirection(text: string, pos: Vec3, offsetDirection: Vec3): void;
    addWithOffsetDirectionFlipTowardEye(text: string, pos: Vec3, offsetDirection: Vec3): void;
    deleteAll(): void;
    readonly labelAppearance: LabelAppearance;
    attachmentLineLength: number;
}

/**
 *
 */
export class PartPoints extends Part {
    private constructor();
    setVertices(vertices: ArrayLike<number>): void;
    pointSize: number;
    color: Color3Like;
}

/**
 *
 */
export class PartTexturedIndexedTriangles extends Part {
    private constructor();
    setMeshData(vertices: ArrayLike<number>, textureCoords: ArrayLike<number>, indices: ArrayLike<number>): void;
    setTexture(texture: Texture): void;
}
} // mrk


//
// sg2
// ===================================================================================
declare namespace sg2 {

/**
 *
 */
export class Grid {
    private constructor();
    name: string;
    id: number;
    gridData: GridData;
    readonly settings: GridSettings;
    transformationMatrix: Mat4;
    customData: any;
    rayIntersect(ray: Ray, hitItem: GridHitItem): boolean;
    readonly quadMeshCount: number;
    getQuadMeshAt(meshIndex: number): QuadMesh;
    getQuadMeshArray(): ReadonlyArray<QuadMesh>;
    addQuadMesh(quadMesh: QuadMesh): void;
}

/**
 *
 */
export class GridData {
    constructor(nodeCountI: number, nodeCountJ: number, elevationArr?: ArrayLike<number>, minElevation?: number, maxElevation?: number);
    readonly nodeCountI: number;
    readonly nodeCountJ: number;
    readonly nodeCount: number;
    readonly cellCountI: number;
    readonly cellCountJ: number;
    readonly cellCount: number;
    readonly visibleCellCount: number;
    spacingI: number;
    spacingJ: number;
    offsetI: number;
    offsetJ: number;
    nodeIndexFromIJ(i: number, j: number): number;
    nodeIJFromIndex(nodeIndex: number): {
        i: number;
        j: number;
    };
    elevationAtPoint(iCoord: number, jCoord: number): number;
    readonly elevationArr: ArrayLike<number>;
    readonly minElevation: number;
    readonly maxElevation: number;
    setElevationArr(elevationArr: ArrayLike<number>, minElevation?: number, maxElevation?: number): void;
    setNodeScalarResultArr(scalarArr: ArrayLike<number>): void;
    readonly nodeScalarResultArr: ArrayLike<number>;
    setCellScalarResultArr(scalarArr: ArrayLike<number>): void;
    readonly cellScalarResultArr: ArrayLike<number>;
    toNodeResultValues(cellResultValues: ArrayLike<number>): Float32Array;
    getScalarResultRange(): Range;
    computeVisibilitiesFromElevations(): void;
    readonly nodeVisibilityArr: ReadonlyArray<boolean>;
    readonly cellVisibilityArr: ReadonlyArray<boolean>;
    rayIntersect(ray: Ray, gridHitItem: GridHitItem): boolean;
    rayIntersectRange(ray: Ray, gridHitItem: GridHitItem, fromI: number, fromJ: number, cellCountI: number, cellCountJ: number): boolean;
    compute2DLineSegmentIntersect(fX1: number, fY1: number, fX2: number, fY2: number): Vec3[];
    static fromElevations(nodeCountI: number, nodeCountJ: number, elevationArr: ArrayLike<number>, minElevation?: number, maxElevation?: number): GridData;
}

/**
 *
 */
export class GridHitItem {
    distanceSquared: number;
    cellIndex: number;
    intersectionPoint: Vec3;
}

/**
 *
 */
export class GridQuery {
    constructor(gridData: GridData);
    setTransformationMatrix(transformationMatrix: Mat4): void;
    computeOutline(): Float32Array;
}

/**
 *
 */
export enum DrawStyle {
    SURFACE = 0,
    SURFACE_MESH = 1,
    SURFACE_OUTLINE_MESH = 2,
    LINES = 3,
    OUTLINE = 4,
}

/**
 *
 */
export class GridSettings {
    private constructor();
    visible: boolean;
    color: Color3Like;
    drawStyle: DrawStyle;
    scalarMapper: ScalarMapperContinuousDomain;
    showPerCellResults: boolean;
}

/**
 *
 */
export interface HitItem {
    gridIndex: number;
    cellIndex: number;
    intersectionPoint: Vec3;
}

/**
 *
 */
export class QuadMesh {
    constructor();
    setMeshData(vertices: ArrayLike<number>, indices: ArrayLike<number>): void;
    visible: boolean;
    color: Color3Like;
    readonly vertexArray: ArrayLike<number>;
    readonly indexArray: ArrayLike<number>;
    setNodeScalarResultArr(scalarArr: ArrayLike<number>): void;
    readonly nodeScalarResultArr: ArrayLike<number>;
}

/**
 *
 */
export class StructGrid2dModel extends Model {
    constructor();
    name: string;
    readonly gridCount: number;
    getGridArray(): ReadonlyArray<Grid>;
    addGrid(): Grid;
    getGridAt(gridIndex: number): Grid;
    getGridById(gridId: number): Grid;
    getBoundingBox(): BoundingBox;
    rayIntersect(ray: Ray): HitItem;
}

/**
 *
 */
export interface RemoteLoaderModelInfo {
    boundingBox: BoundingBox;
}

/**
 *
 */
export class RemoteLoaderStatusInfo {
    totalGridCount: number;
    receivedGridCount: number;
}

/**
 *
 */
export class RemoteLoaderTimingInfo {
    elapsedTime_ms: number;
}

/**
 *
 */
export interface RemoteLoaderInitializationCompleteCallback {
    (modelInfo: RemoteLoaderModelInfo): void;
}

/**
 *
 */
export interface RemoteLoaderCompleteCallback {
    (error: Error): void;
}

/**
 *
 */
export class StructGrid2dModelRemoteLoader {
    constructor(serverUrl: string);
    setInitializationCompleteCallback(initializationCompleteCallback?: RemoteLoaderInitializationCompleteCallback): void;
    setDepthScaleFactor(depthScaleFactor: number): void;
    loadModel(modelKey: string, targetModel: StructGrid2dModel, loadingCompleteCallback?: RemoteLoaderCompleteCallback): void;
    cancelLoading(): void;
    getStatusInfo(): RemoteLoaderStatusInfo;
    getTimingInfo(): RemoteLoaderTimingInfo;
}
} // sg2


//
// ug
// ===================================================================================
declare namespace ug {

/**
 *
 */
export interface AnimationFrameChangedCallback {
    (frameIndex: number): void;
}

/**
 *
 */
export class Animation {
    m_serverErrorOccurred: boolean;
    private constructor();
    targetFrameRate: number;
    particleTraceAnimationStepCount: number;
    runAnimation(run: boolean): void;
    runParticleTraceAnimation(run: boolean): void;
    isModelAnimationRunning(): boolean;
    isAnyAnimationRunning(): boolean;
    setFirstFrame(): void;
    setLastFrame(): void;
    setNextFrame(): void;
    setPreviousFrame(): void;
    setAnimationFrameChangedCallback(animationFrameChangedCallback: AnimationFrameChangedCallback): void;
}

/**
 *
 */
export class CuttingPlane {
    private constructor();
    readonly id: number;
    point: Vec3Like;
    normal: Vec3Like;
    mapScalarResultId: number;
    mapVectorResultId: number;
    vectorGridSpacing: number;
    highlight: boolean;
    clipping: boolean;
    showElementMesh: boolean;
    opacity: number;
    getAsProperties(): CuttingPlaneProperties;
    setFromProperties(props: Partial<CuttingPlaneProperties>): void;
}

/**
 *
 */
export interface CuttingPlaneProperties {
    id: number;
    point: Vec3Like;
    normal: Vec3Like;
    mapScalarResultId: number;
    mapVectorResultId: number;
    vectorGridSpacing: number;
    highlight: boolean;
    clipping: boolean;
    showElementMesh: boolean;
    opacity: number;
}

/**
 *
 */
export class CuttingPlaneUtils {
    static getPositionFromRelativeNormalPosition(relVal: number, normal: Vec3Like, boundingBox: BoundingBox): Vec3;
    static getRelativeNormalPosition(point: Vec3Like, normal: Vec3Like, boundingBox: BoundingBox): number;
}

/**
 *
 */
export class DisplacementSettings {
    private constructor();
    readonly resultId: number;
    scaleFactor: number;
    getAsProperties(): DisplacementSettingsProperties;
    setFromProperties(props: Partial<DisplacementSettingsProperties>): void;
}

/**
 *
 */
export interface DisplacementSettingsProperties {
    resultId: number;
    scaleFactor: number;
}

/**
 *
 */
export enum ObjectType {
    PART = 0,
    CUTTING_PLANE = 1,
    ISOSURFACE = 2,
    ISOVOLUME = 3,
}

/**
 *
 */
export interface HitItem {
    frameIndex: number;
    anythingHit: boolean;
    objectType: ObjectType;
    objectId: number;
    objectIndex: number;
    objectName: string;
    geometryIndex: number;
    elementId: number;
    elementIndex: number;
    elementType: string;
    nodeId: number;
    nodeIndex: number;
    intersectionPoint: Vec3;
    intersectionPointNormal: Vec3;
    elementResult: number;
    elementResultIsInterpolated: boolean;
    nodeResult: number;
    nodeResultIsInterpolated: boolean;
    scalarResultName: string;
    elementTriangleVertices: number[];
}

/**
 *
 */
export class Isosurface {
    private constructor();
    readonly id: number;
    isoValue: number;
    isoScalarResultId: number;
    mapScalarResultId: number;
    mapVectorResultId: number;
    highlight: boolean;
    opacity: number;
    getAsProperties(): IsosurfaceProperties;
    setFromProperties(props: Partial<IsosurfaceProperties>): void;
}

/**
 *
 */
export interface IsosurfaceProperties {
    id: number;
    isoValue: number;
    isoScalarResultId: number;
    mapScalarResultId: number;
    mapVectorResultId: number;
    highlight: boolean;
    opacity: number;
}

/**
 *
 */
export class Isovolume {
    private constructor();
    readonly id: number;
    minimumIsoValue: number;
    maximumIsoValue: number;
    isoScalarResultId: number;
    mapScalarResultId: number;
    mapVectorResultId: number;
    highlight: boolean;
    opacity: number;
    getAsProperties(): IsovolumeProperties;
    setFromProperties(props: Partial<IsovolumeProperties>): void;
}

/**
 *
 */
export interface IsovolumeProperties {
    id: number;
    minimumIsoValue: number;
    maximumIsoValue?: number;
    isoScalarResultId: number;
    mapScalarResultId: number;
    mapVectorResultId: number;
    highlight: boolean;
    opacity: number;
}

/**
 *
 */
export interface IdNamePair {
    readonly id: number;
    readonly name: string;
}

/**
 *
 */
export interface PartInfo {
    readonly geometryIndex: number;
    readonly partId: number;
    readonly name: string;
}

/**
 *
 */
export interface StateInfo {
    readonly id: number;
    readonly name: string;
    readonly referenceValue: number;
}

/**
 *
 */
export interface VTFxCaseInfo {
    readonly caseId: number;
    readonly name: string;
}

/**
 *
 */
export class ModelDirectory {
    readonly stateInfoArray: ReadonlyArray<StateInfo>;
    readonly scalarResultArray: ReadonlyArray<IdNamePair>;
    readonly vectorResultArray: ReadonlyArray<IdNamePair>;
    readonly displacementResultArray: ReadonlyArray<IdNamePair>;
    readonly hasTransformationResult: boolean;
    readonly partInfoArray: ReadonlyArray<PartInfo>;
    readonly vtfxCaseInfoArray: ReadonlyArray<VTFxCaseInfo>;
    private constructor();
    getAllStateIds(): number[];
    getStateNameById(id: number): string;
    getStateReferenceValueById(id: number): number;
    getScalarNameById(id: number): string;
    getVectorNameById(id: number): string;
    getDisplacementNameById(id: number): string;
    getPartNameById(geometryIndex: number, id: number): string;
}

/**
 *
 */
export enum ModeShapeAnimationType {
    QUARTER = 0,
    HALF = 1,
    FULL = 2,
}

/**
 *
 */
export class ModelSpec {
    private constructor();
    stateIdArray: ReadonlyArray<number>;
    fringesResultId: number;
    vectorResultId: number;
    displacementResultId: number;
    useTransformationResult: boolean;
    modeShapeFrameCount: number;
    modeShapeAnimationType: ModeShapeAnimationType;
    getAsProperties(): ModelSpecProperties;
    setFromProperties(props: Partial<ModelSpecProperties>): void;
}

/**
 *
 */
export interface ModelSpecProperties {
    stateIdArray: ReadonlyArray<number>;
    fringesResultId: number;
    vectorResultId: number;
    displacementResultId: number;
    useTransformationResult: boolean;
    modeShapeFrameCount: number;
    modeShapeAnimationType: ModeShapeAnimationType;
}

/**
 *
 */
export enum ParticleTraceVisualizationStyle {
    CYLINDERS = 0,
    SPHERES = 1,
    COMETS = 2,
}

/**
 *
 */
export class ParticleTraceGroup {
    private constructor();
    readonly id: number;
    seedPointArray: ReadonlyArray<Vec3Like>;
    traceBothDirections: boolean;
    vectorResultId: number;
    mapScalarResultId: number;
    visualizationStyle: ParticleTraceVisualizationStyle;
    color: Color3Like;
    radius: number;
    pointSamplingPulseCount: number;
    relativeCometLength: number;
    getAsProperties(): ParticleTraceGroupProperties;
    setFromProperties(props: Partial<ParticleTraceGroupProperties>): void;
}

/**
 *
 */
export interface ParticleTraceGroupProperties {
    id: number;
    seedPointArray: ReadonlyArray<Vec3Like>;
    traceBothDirections: boolean;
    vectorResultId: number;
    mapScalarResultId: number;
    visualizationStyle: ParticleTraceVisualizationStyle;
    color: Color3Like;
    radius: number;
    pointSamplingPulseCount: number;
    relativeCometLength: number;
}

/**
 *
 */
export class ParticleTraceUtils {
    static computeGridPoints(center: Vec3Like, normal: Vec3Like, numU: number, numV: number, groupSpacing: number): Vec3[];
}

/**
 *
 */
export enum DrawStyle {
    SURFACE = 0,
    SURFACE_MESH = 1,
    SURFACE_OUTLINE_MESH = 2,
    LINES = 3,
    POINTS = 4,
    OUTLINE = 5,
    HIDDEN_LINES_REMOVED = 6,
}

/**
 *
 */
export enum EyeLift {
    OFF = 0,
    FACTOR_1 = 1,
    FACTOR_0_1 = 2,
    FACTOR_0_01 = 3,
}

/**
 *
 */
export class PartSettings {
    private constructor();
    readonly geometryIndex: number;
    readonly partId: number;
    visible: boolean;
    highlight: boolean;
    drawStyle: DrawStyle;
    color: Color3Like;
    opacity: number;
    smooth: boolean;
    showFringes: boolean;
    showVectors: boolean;
    eyeLift: EyeLift;
    getAsProperties(): PartSettingsProperties;
    setFromProperties(props: Partial<PartSettingsProperties>): void;
}

/**
 *
 */
export interface PartSettingsProperties {
    geometryIndex: number;
    partId: number;
    visible: boolean;
    highlight: boolean;
    drawStyle: DrawStyle;
    color: Color3Like;
    opacity: number;
    smooth: boolean;
    showFringes: boolean;
    showVectors: boolean;
}

/**
 *
 */
export interface QueryAllElementResultsForFrameData {
    elementScalarValues: number[];
    elementCentroids: Vec3[];
}

/**
 *
 */
export interface QueryAllElementResultsForFrameCallback {
    (data: QueryAllElementResultsForFrameData): void;
}

/**
 *
 */
export class QueryAllElementResultsForFrame extends RemoteQuery {
    constructor(model: RemoteModel);
    executeQuery(frameIndex: number, callback: QueryAllElementResultsForFrameCallback): void;
}

/**
 *
 */
export interface QueryGlobalNodePositionsForFramesDataItem {
    nodeIdent: NodeIdent;
    frameIndices: number[];
    globalNodePositions: Vec3[];
}

/**
 *
 */
export interface QueryGlobalNodePositionsForFramesData {
    nodePositionArr: QueryGlobalNodePositionsForFramesDataItem[];
}

/**
 *
 */
export interface QueryGlobalNodePositionsForFramesCallback {
    (data: QueryGlobalNodePositionsForFramesData): void;
}

/**
 *
 */
export class QueryGlobalNodePositionsForFrames extends RemoteQuery {
    constructor(model: RemoteModel);
    executeQuery(frameIndex: number, nodes: NodeIdent[], callback: QueryGlobalNodePositionsForFramesCallback): void;
}

/**
 *
 */
export interface QueryNodeScalarResultData {
    resultId: number;
    nodeIdent: NodeIdent;
    xValues: number[];
    yValues: number[];
}

/**
 *
 */
export interface QueryNodeScalarResultCallback {
    (data: QueryNodeScalarResultData): void;
}

/**
 *
 */
export class QueryNodeScalarResult extends RemoteQuery {
    constructor(model: RemoteModel);
    executeQuery(resultId: number, nodeIdent: NodeIdent, callback: QueryNodeScalarResultCallback): void;
}

/**
 *
 */
export interface OpenModelCallback {
    (error: Error, model: RemoteModel): void;
}

/**
 *
 */
export interface DataStreamingCompleteCallback {
    (model: RemoteModel): void;
}

/**
 *
 */
export interface RayIntersectResult {
    model: RemoteModel;
    hitItem: HitItem;
}

/**
 *
 */
export interface RayIntersectCallback {
    (intersectResult: RayIntersectResult): void;
}

/**
 *
 */
export enum StreamingStatus {
    DATA_STREAMING_NOT_STARTED = 0,
    STREAMING_DATA = 1,
    DATA_STREAMING_COMPLETE = 2,
}

/**
 *
 */
export interface ProgressData {
    streamingStatus: StreamingStatus;
    pctDone: number;
    pctTotalDataComplete: number;
    debugMessage: string;
}

/**
 *
 */
export interface ProgressHandler {
    (progressData: ProgressData): void;
}

/**
 *
 */
export interface ResultRangeChangedHandler {
    (model: RemoteModel): void;
}

/**
 *
 */
export interface ServerErrorHandler {
    (err: Error, model: RemoteModel): void;
}

/**
 *
 */
export interface NodeIdent {
    geometryIndex: number;
    partId: number;
    nodeIndex: number;
}

/**
 *
 */
export interface SendToCloudCallback {
    (error: Error, modelUrl: string): void;
}

/**
 *
 */
export interface SendToCloudConfig {
    cloudPortalUrl: string;
    uploadId: string;
    caseName: string;
}

/**
 *
 */
export class RemoteModel extends Model {
    constructor(socket: SocketIOClient.Socket);
    static getCustomServerInfo(socket: SocketIOClient.Socket, callback: (customData: any) => void): void;
    name: string;
    readonly modelDirectory: ModelDirectory;
    readonly modelInfoString: string;
    readonly animation: Animation;
    readonly modelSpec: ModelSpec;
    readonly frameCount: number;
    currentFrameIndex: number;
    readonly currentStepText: string;
    readonly partCount: number;
    readonly scalarSettingsCount: number;
    readonly vectorSettingsCount: number;
    readonly displacementSettingsCount: number;
    readonly cuttingPlaneCount: number;
    readonly isosurfaceCount: number;
    readonly isovolumeCount: number;
    readonly particleTraceGroupCount: number;
    openModel(modelKey: string, openCompletedCallback?: OpenModelCallback, configFlags?: string): void;
    closeModel(): void;
    reloadModel(reloadCompletedCallback?: OpenModelCallback): void;
    getBoundingBox(): BoundingBox;
    setServerErrorHandler(errorHandler: ServerErrorHandler): void;
    setProgressHandler(progressHandler: ProgressHandler): void;
    setResultRangeChangedHandler(resultRangeChangedHandler: ResultRangeChangedHandler): void;
    getScalarResultMinimumValue(resultId: number): number;
    getScalarResultMaximumValue(resultId: number): number;
    enablePreloading(enable: boolean): void;
    isPreloadingEnabled(): boolean;
    setOneShotDataStreamingCompleteCallback(dataStreamingCompleteCallback: DataStreamingCompleteCallback): void;
    rayIntersect(frameIndex: number, ray: Ray, rayIntersectCallback: RayIntersectCallback): void;
    getPartSettingsArray(): ReadonlyArray<PartSettings>;
    getPartSettingsById(geometryIndex: number, partId: number): PartSettings;
    getPartSettingsAt(partIndex: number): PartSettings;
    setDrawStyleAllParts(drawStyle: DrawStyle, opacity?: number): void;
    getScalarSettingsArray(): ReadonlyArray<ScalarSettings>;
    getScalarSettingsAt(resultIndex: number): ScalarSettings;
    getScalarSettingsById(resultId: number): ScalarSettings;
    getVectorSettingsArray(): ReadonlyArray<VectorSettings>;
    getVectorSettingsAt(resultIndex: number): VectorSettings;
    getVectorSettingsById(resultId: number): VectorSettings;
    getDisplacementSettingsArray(): ReadonlyArray<DisplacementSettings>;
    getDisplacementSettingsAt(resultIndex: number): DisplacementSettings;
    getDisplacementSettingsById(resultId: number): DisplacementSettings;
    addCuttingPlane(): CuttingPlane;
    getCuttingPlaneArray(): ReadonlyArray<CuttingPlane>;
    getCuttingPlaneAt(index: number): CuttingPlane;
    getCuttingPlaneById(id: number): CuttingPlane;
    deleteCuttingPlaneById(id: number): void;
    deleteAllCuttingPlanes(): void;
    addIsosurface(): Isosurface;
    getIsosurfaceArray(): ReadonlyArray<Isosurface>;
    getIsosurfaceAt(index: number): Isosurface;
    getIsosurfaceById(id: number): Isosurface;
    deleteIsosurfaceById(id: number): void;
    deleteAllIsosurfaces(): void;
    addIsovolume(): Isovolume;
    getIsovolumeArray(): ReadonlyArray<Isovolume>;
    getIsovolumeAt(index: number): Isovolume;
    getIsovolumeById(id: number): Isovolume;
    deleteIsovolumeById(id: number): void;
    deleteAllIsovolumes(): void;
    addParticleTraceGroup(): ParticleTraceGroup;
    getParticleTraceGroupArray(): ReadonlyArray<ParticleTraceGroup>;
    getParticleTraceGroupAt(index: number): ParticleTraceGroup;
    getParticleTraceGroupById(id: number): ParticleTraceGroup;
    deleteParticleTraceGroupById(id: number): void;
    deleteAllParticleTraceGroups(): void;
    sendToCloud(view: View, config: SendToCloudConfig, callback: SendToCloudCallback): void;
    clientSideRayIntersect(ray: Ray): Vec3;
}

/**
 *
 */
export abstract class RemoteQuery {
}

/**
 *
 */
export enum ColorScheme {
    NORMAL = 0,
    NORMAL_INVERTED = 1,
    BLACK_TO_WHITE = 2,
    WHITE_TO_BLACK = 3,
    GREEN_TO_BROWN = 4,
    WHITE_TO_BROWN = 5,
    METAL_CASTING = 6,
    THERMAL_1 = 7,
    THERMAL_2 = 8,
    THERMAL_3 = 9,
}

/**
 *
 */
export enum AutoRangeMode {
    ALL_ITEMS = 0,
    VISIBLE_ITEMS = 1,
}

/**
 *
 */
export enum LegendColorMarkerPosition {
    ABOVE = 0,
    BELOW = 1,
}

/**
 *
 */
export class LegendColorMarker {
    position: LegendColorMarkerPosition;
    color: Color3Like;
    description: string;
}

/**
 *
 */
export enum LegendVisibilityMode {
    NEVER = 0,
    AUTO = 1,
    ALWAYS = 2,
}

/**
 *
 */
export class ScalarSettings {
    private constructor();
    readonly resultId: number;
    readonly rangeMinimum: number;
    readonly rangeMaximum: number;
    readonly isAutoRangeEnabled: boolean;
    readonly autoRangeMode: AutoRangeMode;
    logarithmicMapping: boolean;
    nodeAveragedValues: boolean;
    colorScheme: ColorScheme;
    aboveRangeColor: Color3Like;
    belowRangeColor: Color3Like;
    undefinedColor: Color3Like;
    levelCount: number;
    scalingConstantTerm: number;
    scalingFirstDegreeTerm: number;
    colorMarkerArray: ReadonlyArray<LegendColorMarker>;
    legendVisibilityMode: LegendVisibilityMode;
    setRange(min: number, max: number): void;
    setAutoRange(mode: AutoRangeMode): void;
    getAsProperties(): ScalarSettingsProperties;
    setFromProperties(props: Partial<ScalarSettingsProperties>): void;
}

/**
 *
 */
export interface ScalarSettingsProperties {
    resultId: number;
    autoRangeMode: AutoRangeMode;
    rangeMinimum: number;
    rangeMaximum: number;
    logarithmicMapping: boolean;
    nodeAveragedValues: boolean;
    colorScheme: ColorScheme;
    aboveRangeColor: Color3Like;
    belowRangeColor: Color3Like;
    undefinedColor: Color3Like;
    levelCount: number;
    scalingConstantTerm: number;
    scalingFirstDegreeTerm: number;
    colorMarkerArray: LegendColorMarker[];
    legendVisibilityMode: LegendVisibilityMode;
}

/**
 *
 */
export enum VectorColorMode {
    SINGLE = 0,
    COLOR_BY_FRINGES = 1,
}

/**
 *
 */
export enum VectorScaleMode {
    ABSOLUTE_SCALING = 0,
    RELATIVE_SCALING = 1,
}

/**
 *
 */
export class VectorSettings {
    private constructor();
    readonly resultId: number;
    scaleMode: VectorScaleMode;
    scaleFactor: number;
    colorMode: VectorColorMode;
    singleVectorColor: Color3Like;
    readonly isClampingEnabled: boolean;
    readonly clampingRangeMinimum: number;
    readonly clampingRangeMaximum: number;
    readonly isFilteringEnabled: boolean;
    readonly filteringRangeMinimum: number;
    readonly filteringRangeMaximum: number;
    setClampingRange(clampMinLength: number, clampMaxLength: number): void;
    disableClamping(): void;
    setFilteringRange(minLength: number, maxLength: number): void;
    disableFiltering(): void;
    getAsProperties(): VectorSettingsProperties;
    setFromProperties(props: Partial<VectorSettingsProperties>): void;
}

/**
 *
 */
export interface VectorSettingsProperties {
    resultId: number;
    scaleMode: VectorScaleMode;
    scaleFactor: number;
    colorMode: VectorColorMode;
    singleVectorColor: Color3Like;
    enableClamping: boolean;
    clampingRangeMinimum: number;
    clampingRangeMaximum: number;
    enableFiltering: boolean;
    filteringRangeMinimum: number;
    filteringRangeMaximum: number;
}
} // ug


//
// usg
// ===================================================================================
declare namespace usg {

/**
 *
 */
export class DisplacementSettings {
    scaleFactor: number;
}

/**
 *
 */
export class ElementHighlighter {
    constructor(usgModel: UnstructGridModel, markupModel: mrk.MarkupModel);
    addElementHighlight(surfaceColor: Color3, state: State, partIndex: number, elementIndex: number): boolean;
    addElementHighlight(surfaceColor: Color3, hitItem: HitItem): boolean;
    addElementLabel(state: State, partIndex: number, elementIndex: number, label: string): boolean;
    addElementLabel(hitItem: HitItem): boolean;
    private private_getElementTriangleTessellation(state, partIndex, elementIndex);
}

/**
 *
 */
export class Geometry {
    readonly partCount: number;
    getPartArray(): ReadonlyArray<Part>;
    addPart(): Part;
    getPartAt(partIndex: number): Part;
    deletePartAt(partIndex: number): void;
    deleteAllParts(): void;
}

/**
 *
 */
export class HitItem {
    stateIndex: number;
    partIndex: number;
    partName: string;
    elementId: number;
    elementIndex: number;
    elementNodeCount: number;
    closestNodeId: number;
    closestNodeIndex: number;
    intersectionPoint: Vec3;
    intersectionPointNormal: Vec3;
}

/**
 *
 */
export interface OptionalMeshData {
    noPreserveSharpEdges?: boolean;
    nodeIdArray?: ArrayLike<number>;
    elementIdArray?: ArrayLike<number>;
}

/**
 *
 */
export class Mesh {
    constructor(nodeArr: ArrayLike<number>, singleElementType: number, elementNodeIndexArr: ArrayLike<number>, optionalData?: OptionalMeshData);
    constructor(nodeArr: ArrayLike<number>, elementTypeArr: ArrayLike<number>, elementNodeIndexArr: ArrayLike<number>, optionalData?: OptionalMeshData);
    readonly nodeCount: number;
    readonly nodeArray: ArrayLike<number>;
    readonly elementCount: number;
    readonly elementNodeIndexArray: ArrayLike<number>;
    readonly singleElementType: number;
    readonly elementTypeArray: ArrayLike<number>;
    getBoundingBox(): BoundingBox;
}

/**
 *
 */
export class MeshQuery {
    constructor(mesh: Mesh);
    constructor(state: State, partIndex: number);
    getElementNodeIndices(elementIndex: number): ArrayLike<number>;
    getNodePosition(nodeIndex: number): Vec3;
    getElementCentroid(elementIndex: number): Vec3;
    getElementSurfaceNormal(elementIndex: number): Vec3;
}

/**
 *
 */
export class Part {
    private constructor();
    name: string;
    mesh: Mesh;
    readonly settings: PartSettings;
    customData: any;
}

/**
 *
 */
export class PartDisplacements {
    constructor(displacementsArr: ArrayLike<number>);
    readonly displacementArray: ArrayLike<number>;
}

/**
 *
 */
export class PartScalars {
    constructor(resultMapping: ResultMapping, valueArrray: ArrayLike<number>);
    readonly valueArray: ArrayLike<number>;
    readonly resultMapping: ResultMapping;
    getValueRange(): Range;
}

/**
 *
 */
export enum DrawStyle {
    SURFACE = 0,
    SURFACE_MESH = 1,
    SURFACE_OUTLINE_MESH = 2,
    LINES = 3,
    POINTS = 4,
    OUTLINE = 5,
    HIDDEN_LINES_REMOVED = 6,
}

/**
 *
 */
export class PartSettings {
    private constructor();
    visible: boolean;
    highlight: boolean;
    smooth: boolean;
    drawStyle: DrawStyle;
    color: Color3Like;
    opacity: number;
    fringesVisible: boolean;
    vectorsVisible: boolean;
    displacementVisible: boolean;
}

/**
 *
 */
export class PartVectors {
    constructor(resultMapping: ResultMapping, vectorsArr: ArrayLike<number>);
    readonly vectorArray: ArrayLike<number>;
    readonly resultMapping: ResultMapping;
    getVectorLengthRange(): Range;
}

/**
 *
 */
export class ScalarSettings {
    name: string;
    scalarMapper: ScalarMapperContinuousDomain;
}

/**
 *
 */
export class State {
    private constructor();
    name: string;
    referenceValue: number;
    geometry: Geometry;
    setPartFringesAt(partIndex: number, partFringes: PartScalars): void;
    removeAllPartFringes(): void;
    getPartFringesAt(partIndex: number): PartScalars;
    getFringesResultRange(): Range;
    setPartVectorsAt(partIndex: number, partVectors: PartVectors): void;
    removeAllPartVectors(): void;
    getPartVectorsAt(partIndex: number): PartVectors;
    getVectorLengthRange(): Range;
    setPartDisplacementsAt(partIndex: number, partDisplacements: PartDisplacements): void;
    removeAllPartDisplacements(): void;
    getPartDisplacementsAt(partIndex: number): PartDisplacements;
    setPartTransformationAt(partIndex: number, partTransformationMatrix: Mat4): void;
    getPartTransformationAt(partIndex: number): Mat4;
    removeAllPartTransformations(): void;
    getBoundingBox(): BoundingBox;
}

/**
 *
 */
export enum ResultMapping {
    PER_NODE = 0,
    PER_ELEMENT = 1,
    PER_ELEMENT_NODE = 2,
}

/**
 *
 */
export enum VizStateStrategy {
    VIZ_STATES_SINGLE = 0,
    VIZ_STATES_ALL_SINGLE_GPU = 1,
    VIZ_STATES_ALL = 2,
}

/**
 *
 */
export interface UnstructGridModelOptions {
    vizStateStrategy: VizStateStrategy;
}

/**
 *
 */
export class UnstructGridModel extends Model {
    constructor(options?: UnstructGridModelOptions);
    name: string;
    readonly stateCount: number;
    currentStateIndex: number;
    readonly currentState: State;
    readonly fringesSettings: ScalarSettings;
    readonly vectorSettings: VectorSettings;
    readonly displacementSettings: DisplacementSettings;
    getStateArray(): ReadonlyArray<State>;
    addState(): State;
    deleteStateAt(stateIndex: number): void;
    deleteStateRange(startIndex: number, deleteCount?: number): void;
    deleteAllStates(): void;
    getStateAt(stateIndex: number): State;
    getFringesResultRange(): Range;
    rayIntersect(ray: Ray): HitItem;
    getBoundingBox(): BoundingBox;
}

/**
 *
 */
export enum ScaleMode {
    ABSOLUTE_SCALING = 0,
    RELATIVE_SCALING = 1,
}

/**
 *
 */
export class VectorSettings {
    name: string;
    singleVectorColor: Color3;
    readonly scaleFactor: number;
    readonly scaleMode: ScaleMode;
    setScaleFactor(mode: ScaleMode, scaleFactor: number): void;
}
} // usg


//
// utils
// ===================================================================================
declare namespace utils {

/**
 *
 */
export class BoxGenerator {
    constructor();
    setOriginAndExtent(origin: Vec3, extent: Vec3): void;
    setCenterAndExtent(center: Vec3, extent: Vec3): void;
    setOrientation(rightDir: Vec3, upDir: Vec3): void;
    generate(): void;
    getVertices(): Vec3[];
    getVerticesNumber(): number[];
    getIndices(): number[];
}

/**
 *
 */
export class GeometryModelMockups {
    static populateModelWithSingleTriangleParts(model: geo.GeometryModel, numParts: number): void;
    static populateModelWithTexturedQuadParts(model: geo.GeometryModel, numParts: number, texture: Texture): void;
    static populateModelWithLineSquaresParts(model: geo.GeometryModel, numParts: number): void;
}

/**
 *
 */
export class RemoteModelPickHelper {
    constructor(viewer: Viewer);
    setPickResultCallback(pickResultCallback: ug.RayIntersectCallback): void;
    private handlePick(phd);
}

/**
 *
 */
export class RemoteModelHighlightHelper {
    constructor(model: ug.RemoteModel);
    setMultiHighlight(multiHighlight: boolean): void;
    clearAll(): void;
    add(objectType: ug.ObjectType, objectId: number, geometryIndex: number): void;
    toggle(objectType: ug.ObjectType, objectId: number, geometryIndex: number): void;
    remove(objectType: ug.ObjectType, objectId: number, geometryIndex: number): void;
}
} // utils


//
// cug
// ===================================================================================
declare namespace cug {

/**
 *
 */
export class Animation {
    private constructor();
    targetFrameRate: number;
    particleTraceAnimationStepCount: number;
    runAnimation(run: boolean): void;
    runParticleTraceAnimation(run: boolean): void;
    setFirstFrame(): void;
    setLastFrame(): void;
    setNextFrame(): void;
    setPreviousFrame(): void;
}

/**
 *
 */
export enum ColorLegendType {
    UNIFORM_LEVELS = 0,
    CONTINUOUS = 1,
}

/**
 *
 */
export class ColorLegend {
    private constructor();
}

/**
 *
 */
export interface InitializationCompleteInfo {
    boundingBox: BoundingBox;
    defaultCamera?: {
        eye: Vec3Like;
        direction: Vec3Like;
        up: Vec3Like;
        fieldOfViewYDeg?: number;
        frontPlaneFrustumHeight?: number;
    };
}

/**
 *
 */
export interface InitializationCompleteCallback {
    (error: Error, info: InitializationCompleteInfo): void;
}

/**
 *
 */
export interface ProgressData {
    pctDone: number;
    debugMessage: string;
}

/**
 *
 */
export interface ProgressHandler {
    (progressData: ProgressData): void;
}

/**
 *
 */
export interface RayIntersectCallback {
    (hitItem: HitItem): void;
}

/**
 *
 */
export class ConstantRemoteModel extends Model implements ProcessableModel {
    constructor(serverUrl: string);
    name: string;
    readonly partCount: number;
    readonly cuttingPlaneCount: number;
    readonly isosurfaceCount: number;
    readonly isovolumeCount: number;
    readonly particleTraceGroup: number;
    readonly frameCount: number;
    currentFrameIndex: number;
    readonly animation: Animation;
    setCurrentParticleTraceTime(time: number, isFirstParticleTraceCycle: boolean): void;
    getCurrentParticleTraceTime(): number;
    isFirstParticleTraceCycle(): boolean;
    enablePreLoading(enable: boolean): void;
    setGuidingCamera(camera: Camera): void;
    setProgressHandler(progressHandler: ProgressHandler): void;
    openModel(modelKey: string, initializationCompleteCallback: InitializationCompleteCallback): void;
    closeModel(): void;
    getFrameInfoArray(): ReadonlyArray<FrameInfo>;
    getFrameInfoAt(index: number): FrameInfo;
    getPartArray(): ReadonlyArray<Part>;
    getPartAt(index: number): Part;
    getCuttingPlaneArray(): ReadonlyArray<CuttingPlane>;
    getCuttingPlaneAt(index: number): CuttingPlane;
    getIsosurfaceArray(): ReadonlyArray<Isosurface>;
    getIsosurfaceAt(index: number): Isosurface;
    getIsovolumeArray(): ReadonlyArray<Isovolume>;
    getIsovolumeAt(index: number): Isovolume;
    getParticleTraceGroupArray(): ReadonlyArray<ParticleTraceGroup>;
    getParticleTraceGroupAt(index: number): ParticleTraceGroup;
    rayIntersect(ray: Ray, view: View): HitItem;
    rayIntersectServerInfo(ray: Ray, view: View, rayIntersectCallback: RayIntersectCallback): void;
    getBoundingBox(): BoundingBox;
}

/**
 *
 */
export class CuttingPlane {
    private constructor();
    visible: boolean;
    highlight: boolean;
}

/**
 *
 */
export class FrameInfo {
    private constructor();
    readonly stateId: number;
    readonly name: string;
    readonly referenceValue: number;
}

/**
 *
 */
export class HitItem {
    frameIndex: number;
    objectType: ObjectType;
    objectIndex: number;
    nodeId: number;
    nodeIndex: number;
    nodeScalarResult: number;
    elementId: number;
    elementIndex: number;
    elementType: string;
    elementLocalSurface: number;
    elementScalarResult: number;
    intersectionPoint: Vec3;
    intersectionPointNormal: Vec3;
    elementTriangleVertices: number[];
}

/**
 *
 */
export class Isosurface {
    private constructor();
    visible: boolean;
    highlight: boolean;
}

/**
 *
 */
export class Isovolume {
    private constructor();
    visible: boolean;
    highlight: boolean;
}

/**
 *
 */
export enum ObjectType {
    PART = 0,
    CUTTING_PLANE = 1,
    ISOSURFACE = 2,
    ISOVOLUME = 3,
}

/**
 *
 */
export class Part {
    private constructor();
    readonly geometryIndex: number;
    readonly id: number;
    readonly name: string;
    visible: boolean;
    surfaceMesh: boolean;
    highlight: boolean;
}

/**
 *
 */
export class ParticleTraceGroup {
    private constructor();
    readonly seedPointArray: ReadonlyArray<Vec3Like>;
}
} // cug
