import { getJSONResponse } from '../utils/util'
import { getOutputNames, rootAPI, startProcessorRequest } from '../api/APIHelper'
import { createProcessorInputFormData, createProcessorOutputFormData } from '../api/formDataCreator'
import { extractProcessorIOs } from '../api/extractor'

const processorEndpoint = rootAPI + '/processors/'

export default {
  props: {
    processorID: String
  },
  data: () => ({
    isStarted: true,
    startParams: {},
    processor: {},
    dataSourceChannels: [],
    isLoading: false
  }),
  watch: {
    processorID: {
      immediate: true,
      handler (processorID) {
        if (processorID !== '') {
          this.loadProcessor(processorID)
        }
      }
    }
  },
  methods: {
    async startProcessor () {
      let formData = this.getStartForm()
      try {
        this.isLoading = true
        const response = await startProcessorRequest(formData)
        if (!response.ok) {
          this.$root.displayPopup('Server error: ' + response.statusText)
          this.isLoading = false
          return
        }
        this.$root.displayPopup('Process Started!')
        // Get back to frontpage as you're finished
        this.$router.push({ name: 'Home' })
      } catch (error) {
        this.$root.displayPopup('Network Error' + error)
      }
      this.isLoading = false
    },
    getStartForm () {
      let formData = new FormData()
      formData = createProcessorInputFormData(this.processor.inputs)
      formData = createProcessorOutputFormData(this.processor.outputs, formData)
      formData.append('id', this.processorID)
      formData.append('start_params', JSON.stringify(this.startParams))
      return formData
    },
    async findStart (processorJSON) {
      let newParams = {}
      const blueprint = processorJSON.blueprint_id
      const startParamResponse = await getJSONResponse(
        rootAPI + '/blueprints/' + blueprint
      )
      startParamResponse.start_params.forEach(element => {
        newParams[element.name] = element.default
      })
      this.startParams = newParams
    },
    async loadProcessor (processorID) {
      const processorJSON = await getJSONResponse(
        processorEndpoint + processorID
      )
      const processorIOs = extractProcessorIOs(processorJSON)
      if (!processorJSON.started) {
        this.findStart(processorJSON)
        this.isStarted = false
      }
      if (processorIOs) {
        this.processor = processorIOs
        this.dataSourceChannels = await getOutputNames(
          '/topics/' + processorJSON.source_topic
        )
      }
    }
  }
}
