import { lazyLoadView } from '../utils/vueutils'
import LayoutSaver from '../store/custom/dashboardLayoutSaver'

// Put routes that should be displayed in the navigation menu here
export const navRoutes = [
  {
    path: '/',
    name: 'Home',
    component: () => lazyLoadView(import('../views/Home'))
  },
  {
    path: '/dashboard',
    component: () => lazyLoadView(import('../views/Dashboard')),
    redirect: () => {
      // Redirect to new dashboard if there is no last selected layout, else load last selected layout
      const lastSelection = LayoutSaver.getLastSelection()
      if (!lastSelection) {
        return { name: 'NewDashboard' }
      }
      return { name: 'DashboardLayout', params: { layoutId: lastSelection } }
    },
    name: 'Dashboard',
    children: [
      {
        path: 'layout/:layoutId',
        name: 'DashboardLayout',
        component: () => lazyLoadView(import('@components/DashboardGrid')),
        props: true
      },
      {
        path: 'newlayout',
        name: 'NewDashboard',
        component: () => lazyLoadView(import('@components/DashboardGrid')),
        props: { isCreatingNewLayout: true, layoutId: 'new layout' }
      }
    ]
  },
  {
    path: '/datasources',
    name: 'Datasources',
    component: () => lazyLoadView(import('../views/DataSourceEditor'))
  },
  {
    path: '/Processors',
    component: () => lazyLoadView(import('../views/Processor')),
    children: [
      {
        path: '',
        name: 'Processors',
        component: () => lazyLoadView(import('../views/Processors'))
      },
      {
        path: 'create',
        name: 'ProcessorCreate',
        component: () => lazyLoadView(import('../components/forms/ProcessorCreator'))
      },
      {
        path: ':processorID',
        name: 'ProcessorEdit',
        component: () => lazyLoadView(import('../views/ProcessorEdit'))
      }
    ]
  },
  {
    path: '/about',
    name: 'About',
    // route level code-splitting
    // this generates a separate chunk (about.[hash].js) for this route
    // which is lazy-loaded when the route is visited.
    component: () => import(/* webpackChunkName: "about" */ '../views/About.vue')
  }
]
